@@ -9,7 +9,7 @@ from anchorpy.error import AccountDoesNotExistError
 from solana.publickey import PublicKey
 from solana.keypair import Keypair
 from solana.transaction import TransactionInstruction, Transaction, TransactionSignature
-from solana.rpc.commitment import Commitment, Finalized
+from solana.rpc.commitment import Commitment, Confirmed
 from solana.rpc.async_api import AsyncClient
 from solana.rpc.types import TxOpts
 from solana.sysvar import SYSVAR_RENT_PUBKEY
@@ -39,11 +39,13 @@ def GenIxDispatch(cls):
     and adds a trivial wrapper to dispatch the instruction using `Zo.send`.
     The `__doc__` is moved to the generated method.
     """
+
     def gen(f):
         async def g(self, *a, **kw):
             # If you're looking for the source, see the source
             # for the `_ix` variant of this method.
             return await self.send(f(self, *a, **kw))
+
         return g
 
     for n, f in inspect.getmembers(cls, predicate=inspect.isfunction):
@@ -106,6 +108,7 @@ class Zo:
 
     _zo_state: Any
     _zo_state_signer: PublicKey
+    _zo_heimdall_key: PublicKey
     _zo_cache: Any
     _zo_margin: Any
     _zo_margin_key: None | PublicKey
@@ -120,6 +123,7 @@ class Zo:
         _state_signer,
         _margin,
         _margin_key,
+        _heimdall_key,
     ):
         self.__program = _program
         self.__config = _config
@@ -127,6 +131,7 @@ class Zo:
         self._zo_state_signer = _state_signer
         self._zo_margin = _margin
         self._zo_margin_key = _margin_key
+        self._zo_heimdall_key = _heimdall_key
 
     @classmethod
     async def new(
@@ -139,7 +144,7 @@ class Zo:
         create_margin: bool = True,
         tx_opts: TxOpts = TxOpts(
             max_retries=None,
-            preflight_commitment=Finalized,
+            preflight_commitment=Confirmed,
             skip_confirmation=False,
             skip_preflight=False,
         ),
@@ -186,6 +191,8 @@ class Zo:
                 f"Invalid state key ({config.ZO_STATE_ID}) for program id ({config.ZO_PROGRAM_ID})"
             )
 
+        heimdall_key = util.heimdall_pda(program_id=config.ZO_PROGRAM_ID)
+
         margin = None
         margin_key = None
 
@@ -216,8 +223,9 @@ class Zo:
             _state_signer=state_signer,
             _margin=margin,
             _margin_key=margin_key,
+            _heimdall_key=heimdall_key,
         )
-        await zo.refresh(commitment=Finalized)
+        await zo.refresh(commitment=Confirmed)
         return zo
 
     @property
@@ -488,7 +496,7 @@ class Zo:
                 ),
             )
 
-    async def refresh(self, *, commitment: Commitment = Finalized):
+    async def refresh(self, *, commitment: Commitment = Confirmed):
         """Refresh the loaded accounts to see updates."""
         self._zo_state, self._zo_cache, _ = await asyncio.gather(
             self.program.account["State"].fetch(self.__config.ZO_STATE_ID, commitment),
@@ -522,9 +530,9 @@ class Zo:
     def deposit_ix(
         self,
         amount: float,
+        symbol: str,
         /,
         *,
-        mint: PublicKey,
         repay_only: bool = False,
         token_account: None | PublicKey = None,
     ) -> TransactionInstruction:
@@ -532,7 +540,7 @@ class Zo:
 
         Args:
             amount: The amount to deposit, in big units (e.g.: 1.5 SOL, 0.5 BTC).
-            mint: Mint of the collateral being deposited.
+            symbol: Symbol of the collateral being deposited.
             repay_only: If true, will only deposit up to the amount borrowed.
             token_account: The token account to deposit from, defaulting to
                 the associated token account.
@@ -540,12 +548,13 @@ class Zo:
         Returns:
             The transaction signature.
         """
+        info = self.collaterals[symbol]
+        mint = info.mint
+        amount = util.big_to_small_amount(amount, decimals=info.decimals)
+
         if token_account is None:
             token_account = get_associated_token_address(self.wallet.public_key, mint)
 
-        decimals = self.collaterals[mint].decimals
-        amount = util.big_to_small_amount(amount, decimals=decimals)
-
         return self.program.instruction["deposit"](
             repay_only,
             amount,
@@ -557,7 +566,7 @@ class Zo:
                     "authority": self.wallet.public_key,
                     "margin": self._zo_margin_key,
                     "token_account": token_account,
-                    "vault": self.collaterals[mint].vault,
+                    "vault": self.collaterals[symbol].vault,
                     "token_program": TOKEN_PROGRAM_ID,
                 }
             ),
@@ -566,9 +575,9 @@ class Zo:
     def withdraw_ix(
         self,
         amount: float,
+        symbol: str,
         /,
         *,
-        mint: PublicKey,
         allow_borrow: bool = False,
         token_account: None | PublicKey = None,
     ) -> TransactionInstruction:
@@ -586,13 +595,13 @@ class Zo:
         Returns:
             The transaction signature.
         """
+        info = self.collaterals[symbol]
+        mint = info.mint
+        amount = util.big_to_small_amount(amount, decimals=info.decimals)
 
         if token_account is None:
             token_account = get_associated_token_address(self.wallet.public_key, mint)
 
-        decimals = self.collaterals[mint].decimals
-        amount = util.big_to_small_amount(amount, decimals=decimals)
-
         return self.program.instruction["withdraw"](
             allow_borrow,
             amount,
@@ -605,8 +614,9 @@ class Zo:
                     "margin": self._zo_margin_key,
                     "control": self._zo_margin.control,
                     "token_account": token_account,
-                    "vault": self.collaterals[mint].vault,
+                    "vault": self.collaterals[symbol].vault,
                     "token_program": TOKEN_PROGRAM_ID,
+                    "heimdall": self._zo_heimdall_key,
                 }
             ),
         )
@@ -815,3 +825,38 @@ class Zo:
             order_type="reduceonlyioc",
             max_ts=max_ts,
         )
+
+    def cancel_all_orders_ix(self, symbol: str, /, *, limit: int = 128):
+        """Cancel all orders.
+
+        Args:
+            symbol: The market symbol, e.g. "BTC-PERP".
+            limit: Maximum number of orders to cancel.
+
+        Returns:
+            The transaction signature.
+        """
+        info = self.markets[symbol]
+        mkt = self.__dex_markets[symbol]
+        oo = self._get_open_orders_info(symbol).key
+
+        return self.program.instruction["cancel_all_perp_orders"](
+            limit,
+            ctx=Context(
+                accounts={
+                    "authority": self.wallet.public_key,
+                    "state": self.__config.ZO_STATE_ID,
+                    "cache": self._zo_state.cache,
+                    "state_signer": self._zo_state_signer,
+                    "margin": self._zo_margin_key,
+                    "control": self._zo_margin.control,
+                    "oo": oo,
+                    "dex_market": info.address,
+                    "req_q": mkt.req_q,
+                    "event_q": mkt.event_q,
+                    "market_bids": mkt.bids,
+                    "market_asks": mkt.asks,
+                    "dex_program": self.__config.ZO_DEX_ID,
+                }
+            ),
+        )
